"""
Define algunos permisos estándar para utilizar a lo largo de toda la
aplicación.
"""
import logging

from rest_framework.permissions import BasePermission
from django.contrib.auth import get_user_model

User = get_user_model()


class IsOwnerAdminOrReadOnly(BasePermission):
    """
    Permite la lectura a través del método ``GET`` para
    cualquier usuario.

    Cualquier otro tipo de operación estará limitada a que el usuario que
    realiza la petición sea usuario administrador o que coincida con el valor
    del atributo ``user`` del objeto.
    """

    def has_object_permission(self, request, view, obj):
        if request.method == "GET":
            return True
        return obj.user == request.user or request.user.is_superuser


class IsOwnerOrAdmin(BasePermission):
    """
    Cualquier operación de acceso estará limitada a que el usuario que
    realiza la petición sea usuario administrador o que coincida con el valor
    del atributo ``user`` del objeto.

    Este permiso comprueba el objeto que recibe, pudiendo ser un ``User``, un
    objeto con la propiedad ``user`` o cualquiera otra cosa, en este caso
    deberíamos devolver ``False`` como tratamiento de error.
    """

    def has_object_permission(self, request, view, obj):
        if isinstance(obj, User):
            user = obj
        elif hasattr(obj, "user"):
            user = obj.user
        else:
            return False
        return request.user.is_authenticated and (
            user == request.user or request.user.is_superuser
        )



from django.db import models

from django.utils.translation import ugettext as _


class TimeStampedModel(models.Model):
    """
    Clase abstracta que proporciona campos de fecha de cración y modificación
    autogestionadas.
    """

    #: Date and time of creation
    created = models.DateTimeField(_("created"), auto_now_add=True, db_index=True)
    #: Date and time of the last update
    modified = models.DateTimeField(_("modified"), auto_now=True, db_index=True)

    class Meta:
        get_latest_by = "modified"
        ordering = ("-modified", "-created")
        abstract = True



from rest_framework.viewsets import ViewSet


class MultiSerializerViewSet(ViewSet):
    """
    Allows using several serializers in a Django REST Framwork Viewset
    based on HTTP verb.

    You should define the attribute `serializers` as a dictionary where
    the key is the action and the value is the serializer class.
    You should define at least the `default` key.

    .. code:: python

        from django_tgb_basics.viewsets import MultiSerializerViewSet
        from .models import MyModel
        from .serializers import (
            BasicSerializer,
            AnotherSerializer
        )

        class MyModelViewSet(MultiSerializerViewSet):
            queryset = MyModel.objects.all()
            serializers = {
                'default': BasicSerializer,
                'create': BasicSerializer,
                'update': BasicSerializer,
                'partial_update': BasicSerializer,
                'retrieve': AnotherSerializer,
            }

    This is based on the following StackOverflow question
    Ref. https://stackoverflow.com/questions/22616973/django-rest-framework-use-different-serializers-in-the-same-modelviewset
    """

    serializers = {"default": None}

    def get_serializer_class(self):
        """
        Returns the serializer based on the HTTP verb (action).
        If no action is defined, returns the default serializer.
        """
        return self.serializers.get(self.action, self.serializers["default"])

